# Demo-Framework-4k

------

This is a set of "tiny demo templates" or, being more accurate, a set of 1k and 4k demos examples and executable graphics examples for Windows, which can be used to create demoscene productions ("intros" in the case of such tiny demos). These are in fact what I used to create the [Elevated](https://www.pouet.net/prod.php?which=52938) and [Hoody](https://www.pouet.net/prod.php?which=85272), and many of the other demos [here](https://iquilezles.org/prods/index.htm).

## Usage

This package contains one project for each one of the most typical technologies used to create such tiny demos: OpenGL with polygons, OpenGL with just shaders (Shadertoy style), DirectX9 with polygons, DirectX9 with just shaders, software rendering (pure pixel based, multithreaded), etc. Some of these demos implement sound playback too.

All of them require Mentor and Blueberry's [Crinkler compressor](https://github.com/runestubbe/Crinkler/releases) in order to link the binary into an self-extracting executable. You should download it on your own, and copy it next to the solution demoFramework4k.sln. The Visual Studio projects are already setup for you to use Crinkler when you build them in Release mode. On Debug mode, on the other hand, there are no size optimizations and there is more error checks.

For shader based productions, you should probably use Ctrl+Alt+Test's [Shader Minifier](https://github.com/laurentlb/shader_minifier) to get those shaders to become as small as possible.

Most projects come with a file called fp.h that contains floating point approximations to many numbers that use less space that the original number - see [this article for details](https://iquilezles.org/www/articles/float4k/float4k.htm).

Lasly, most projects contain a file called main.h where you can do some basic setup for the demo, like the screen resolution.

## Summary of Demo Examples

These are the demo templates you'll find inside this package:

|Project|Category|Size|Rendering|Features|Sound|
|---|---|---|---|---|---|
|i1k_D3D9Shader|1k intro|870 bytes|shader (HLSL)|a 2d gradient|no|
|i1k_OGLShader|1k intro|813 bytes|shader (GLSL)|a 2d tunnel|no|
|i4k_Software|4k intro|939 bytes|software (CPU)|a 2d tunnel|yes|
|i4k_D3D9|4k intro|1214 bytes|directx 9|a 3d cube|yes|
|i4k_OGL|4k intro|1144 bytes|opengl 1.1|a 3d cube|yes|
|i4k_OGLShader|4k intro|1388 bytes|shader (GLSL)|a 3d raytraced sphere|yes|
|g4k_Software|executable gfx|1502 bytes|sofware (CPU - multithreaded)|ambient occlusion|no|
|g4k_OGLShader|executable gfx|892 bytes|shader (GLSL)|mandelbrot set|no
|g4k_OGLShader_multipass|executable gfx|1011 bytes|shader (GLSL)|mandelbrot set|no|


## Requirements

* Visual Studio 2019
* [Crinkler compressor](https://github.com/runestubbe/Crinkler/releases)

## License

* MIT license

## Credits

* [Inigo Quilez](https://iquilezles.org)
